/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2021/06/06
 * Description        : Main program body.
 *********************************************************************************
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * Attention: This software (modified or not) and binary are used for
 * microcontroller manufactured by Nanjing Qinheng Microelectronics.
 *******************************************************************************/

/*
 *@Note
 Single pulse output routine:
 TIM2_CH1(PA0),TIM2_CH2(PA1)
 This routine demonstrates that in single-pulse mode, when a rising edge is detected
 on the TIM2_CH2(PA1) pin, the TIM2_CH1(PA0) outputs positive pulse.

 */
#include "debug.h"

/*********************************************************************
 * @fn      One_Pulse_Init
 *
 * @brief   Initializes TIM1 one pulse.
 *
 * @param   arr - the period value.
 *          psc - the prescaler value.
 *          ccp - the pulse value.
 *
 * @return  none
 */
void One_Pulse_Init(u16 arr, u16 psc, u16 ccp)
{
	GPIO_InitTypeDef GPIO_InitStructure =
	{
			0
	};
	TIM_OCInitTypeDef TIM_OCInitStructure =
	{
			0
	};
	TIM_ICInitTypeDef TIM_ICInitStructure =
	{
			0
	};
	TIM_TimeBaseInitTypeDef TIM_TimeBaseInitStructure =
	{
			0
	};

	RCC_APB2PeriphClockCmd( RCC_APB2Periph_GPIOA , ENABLE );
	RCC_APB1PeriphClockCmd( RCC_APB1Periph_TIM2 , ENABLE );

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init( GPIOA , &GPIO_InitStructure );

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;
	GPIO_Init( GPIOA , &GPIO_InitStructure );

	TIM_TimeBaseInitStructure.TIM_Period = arr;
	TIM_TimeBaseInitStructure.TIM_Prescaler = psc;
	TIM_TimeBaseInitStructure.TIM_ClockDivision = TIM_CKD_DIV1;
	TIM_TimeBaseInitStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseInit( TIM2 , &TIM_TimeBaseInitStructure );
	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM2;
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	TIM_OCInitStructure.TIM_Pulse = ccp;
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
	TIM_OC1Init( TIM2 , &TIM_OCInitStructure );

	TIM_SelectOnePulseMode( TIM2 , TIM_OPMode_Single );
}

void OnePulse(uint16_t width){
	TIM2->CH1CVR = 1;
	TIM2->ATRLR = width+1;
	TIM_Cmd(TIM2, ENABLE);
}

/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
	USART_Printf_Init( 115200 );
	Delay_Init();
	SystemCoreClockUpdate();
	printf( "SystemClk:%d\r\n" , SystemCoreClock );
	printf( "ChipID:%08x\r\n" , DBGMCU_GetCHIPID() );
	One_Pulse_Init( 200 , 48000 - 1 , 100 );
	while(1)
	{
		for (int var = 0; var < 1000; var+=100) {
			Delay_Ms(1000);
			OnePulse(var);
		}
	}
}
